%% --------------------------------------------------------------------------------------------------
% invert_glm_vb - User variational bayes to invert a glm with fixed noise
%                 precision
%
%---------------------------------------------------------------------------------------------
% INPUT:
%       data
%       params
%
% Optional:
%
%--------------------------------------------------------------------------------------------
% OUTPUT: 
%       Result - posterior distribution, model evidence estimate
%
% -------------------------------------------------------------------------------------------
% REFERENCE:
%
% Author:
%     Sudhir Shankar Raman , Nov 2015 
%

%%

function [result] = invert_glm_vb(data,params)

maxIterations = 1000;
[N,D] = size(data.x);
y = data.y;
x = data.x;

% initialize all q distributions for all dimensions
for i=1:D
    q.mu = zeros(D,1);
    q.cov = eye(D);
end

taue = 1/(params.epsilon^2);
taup = params.prior.tau;
currF = -inf;
xtx = data.x'*data.x;
ytx = data.y'*data.x;
for i=1:maxIterations
    
    % compute new q distributions    
    for d=1:D
        q.cov(d,d) = taue*xtx(d,d) +  taup;
        
        q.mu(d) = (1/q.cov(d,d))*taue*0.5*(2*ytx(d) - q.mu'*xtx(d,:)' - ...
                q.mu'*xtx(:,d) + 2*q.mu(d)*xtx(d,d));       
        
    end
    
    C = taue*xtx +  taup*eye(D);
   
    % compute F - negative free energy
    Ftrace(i) = -0.5*N*log(2*pi) + 0.5*N*log(taue) - 0.5*taue*y'*y + taue*q.mu'*x'*y + ...
         + 0.5*D*log(taup) - 0.5*(q.mu'*C*q.mu + trace(C*inv(q.cov)))...
          + 0.5*log(exp(D*1)*det(inv(q.cov)));

     df = Ftrace(i) - currF;
     currF = Ftrace(i);
    % check to stop
     if df < 1e-10
         break;
     end
end

result.q = q;
result.Ftrace = Ftrace;
result.F = Ftrace(end);
