%% --------------------------------------------------------------------------------------------------
% mcmc_script - script to obtain posterior distribution and model evidence
%
%---------------------------------------------------------------------------------------------
% INPUT:
%      
%
% Optional:
%
%--------------------------------------------------------------------------------------------
% OUTPUT: 
%       modelResult
%
% -------------------------------------------------------------------------------------------
% REFERENCE:
%
% Author:
%     Sudhir Shankar Raman , Nov 2015 
%

%%

function [modelResult]=  mcmc_script(demono)

modelResult = [];
if(demono==1)
    [R]=test_convg();
end
if(demono==2)
    modelResult = infer();    
end
%return;

function [modelResult] = infer()

[data,params] = simulate_glm();

% Run multiple chains at different temperatures
nChains = 25;
nIterations = 3000;
nBurnin = 1500;

nparamlist = zeros(nIterations,5,nChains);

% Run multiple chains for Thermodynamic Integration
 for iChain=0:nChains
        params.T = (iChain/nChains)^5;
        iChain
        %---------------- MCMC inversion ------------------------
        [mR(iChain+1)] = invert_glm_mcmc(data,params,nIterations,nBurnin);        
 end    

% Model Evidence - Using Thermodynamic Integration
listLogLk1 = zeros(nChains+1,2);
for iChain=1:(nChains+1)
    listLogLk1(iChain,:) = [mean(mR(iChain).llk(nBurnin:end)) mR(iChain).T];
end

listLogLk1 = sortrows(listLogLk1,2);
[listModelEvidence] = model_evidence_TI_trace(listLogLk1,ceil(nChains/10));
mR(nChains+1).modelEvidence = listModelEvidence(end,2);
mR(nChains+1).listModelEvidence = listModelEvidence;
mR(nChains+1).paramExpectation = mean(mR(nChains+1).param.beta,2);

modelResult = mR(nChains+1);
modelResult.data = data;
modelResult.params = params;

plot_results(modelResult,params,2);

function [R]= test_convg()

% Simulate Data
[data,params] = simulate_glm();

% Run multiple chains at different temperatures
nChains = 10;
nIterations = 3000;
nBurnin = 1500;

nparamlist = zeros(nIterations,5,nChains);

% Convergence test using Gelman-Rubin test
 for iChain=1:nChains
        iChain
        %---------------- MCMC inversion ------------------------
        [result] = invert_glm_mcmc(data,params,nIterations,nBurnin);
        nparamlist(:,:,iChain) = result.param.beta';
 end    


[R] = psrf(nparamlist)
plot_results(result,params,1);
 

