%% --------------------------------------------------------------------------------------------------
% model_evidence_TI_trace -  calculates model evidence estimate from
% mulitple chains using thermodynamic integration
%
%---------------------------------------------------------------------------------------------
% INPUT:
%       listLogLk   - Nx2 vector where column 1 is the expected log
%       likelihood for a particular chain and column 2 is the temperature
%       of that chain. (N is the number of chains with temperatures ranging
%       from 0 to 1). 
%       startPoint - minimum spacing for the chains (usually 10)       
%
% Optional:
%
%--------------------------------------------------------------------------------------------
% OUTPUT:
%           logEvidence list - list of log evidence approximations based on
%           taking different number of chains. The list is to see how the 
%           value converges. The last value of the list will be the best 
%           approximation since it will consider all the chains. 
% -------------------------------------------------------------------------------------------
% REFERENCE:
%
% Author:   Sudhir Shankar Raman, TNU, UZH & ETHZ - 2014
%
% This file is part of TAPAS, which is released under the terms of the GNU General Public
% Licence (GPL), version 3. For further details, see <http://www.gnu.org/licenses/>. 

%%

function [listModelEvidence] = tapas_model_evidence_TI_trace(listLogLk,startPoint)

if(~exist('startPoint','var'))
    startPoint = 10;
end

listLogLk = sortrows(listLogLk,2);
nChains = length(listLogLk);
N = round(nChains/1);
listModelEvidence = zeros(N-startPoint+1,2);
for i= startPoint:N
    
    tempIndex = round(linspace(1,nChains,i));
    chainResults = listLogLk(tempIndex,:);
    [nChains1 nVariables] = size(chainResults);
    logModelEvidence = 0;

    % numerical integration using the Trapezoid rule.
    for iChain1 = 1:(nChains1 - 1)
        logModelEvidence = logModelEvidence + 0.5*(chainResults(iChain1+1,2) - chainResults(iChain1,2))...
                            *(chainResults(iChain1+1,1) + chainResults(iChain1,1));
    end    

    listModelEvidence(i-startPoint+1,:) = [i logModelEvidence];
end
