%% --------------------------------------------------------------------------------------------------
% simulate_glm - simulates data from a linear model with fixed gaussian
% error
%
%---------------------------------------------------------------------------------------------
% INPUT:
%      
%
% Optional:
%
%--------------------------------------------------------------------------------------------
% OUTPUT: 
%       data - y and X 
%       params - true parameters, exact posterior, prior
%
% -------------------------------------------------------------------------------------------
% REFERENCE:
%
% Author:
%     Sudhir Shankar Raman , Nov 2015 
%

%%

function [data,params] = simulate_glm()

params.beta = [0.8 -0.3 1.5 3 -0.9 ]';

N = 10;
%N= 50;
D = length(params.beta);
data.x = 1*rand(N,D);
yexact = data.x*params.beta;

params.epsilon = sqrt(0.1);

data.y =  params.epsilon*randn(N,1) + yexact;

params.prior.mu = zeros(D,1);
params.prior.tau = 1;
params.prior.a = 1;
params.prior.b = 2;

% ML estimate
betahat = pinv(data.x'*data.x)*data.x'*data.y;
params.exact.tau = params.prior.tau*eye(D) + ...
    (data.x'*data.x)/(params.epsilon^2);

taue = 1/(params.epsilon^2);
params.exact.mu = pinv(params.exact.tau)* ...
    (((data.x'*data.x)/(params.epsilon^2))*betahat + ...
    params.prior.tau*eye(D)* params.prior.mu);

% log evidence
params.exact.logevidence = -0.5*N*log(2*pi) + 0.5*N*log(det(taue)) +  ...
     -0.5*log(det(params.exact.tau))+ 0.5*log(det(params.prior.tau*eye(D))) ;
 
params.exact.logevidence = params.exact.logevidence -0.5*taue*data.y'*data.y ; 
params.exact.logevidence = params.exact.logevidence  + 0.5*taue^2* ...
                        data.y'* data.x*pinv(params.exact.tau)*data.x'*data.y;
params.T = 1;
